import BuildableResource from "./contracts/BuildableResource";
import ResourceFactory from "./contracts/ResourceFactory";
import JSONConvertible from "./contracts/JSONConvertible";
/**
 * This is used to define how a class needs to be constructed from an object.
 */
export default class Builder<ResultType extends BuildableResource<ResultType>> implements JSONConvertible {
    /**
     * The built object to populate.
     */
    private baseObject;
    /**
     * List of parameters that need to be ignored.
     */
    private ignores;
    /**
     * Collection of transformers. Each transformer is a callback function associated to a property,
     * this gets called just before the property is assigned to the {@link baseObject}
     */
    private transformers;
    /**
     * Collection of aliases. Each alias represents an alternative name for the property
     * in the incoming object.
     */
    private aliases;
    /**
     * Collection of list element constructors. This is used to define how to construct
     * the individual items of an incoming list.
     */
    private listElementConstructors;
    /**
     * Instantiates the given class thanks to a {@link ResourceFactory}.
     *
     * @param ctor The class that needs to be instantiated.
     */
    constructor(ctor: ResourceFactory<ResultType>);
    /**
     * Check if this builder is using a specific
     * class to build the object from.
     *
     * @param specifiedClass The class in question
     * @returns True if it's using it, false otherwise.
     */
    isUsingClass(specifiedClass: any): boolean;
    /**
     * Update base object of current builder and instantiates the given class thanks to a {@link ResourceFactory}.
     *
     * @param ctor The class that needs to be instantiated.
     */
    updateBaseObject(ctor: ResourceFactory<ResultType>): void;
    /**
     * Add an ignore directive for one or more paths.
     *
     * @param paths The object paths to ignore when instantiating.
     * @returns The builder.
     */
    ignore(...paths: string[]): this;
    /**
     * Add a transform directive for a given path.
     *
     * @param localPath The property of the typed object that needs to be transformed.
     * @param transformer The {@link ValueTransformer} used.
     * @returns This builder.
     */
    transform(localPath: string, transformerIn: Action, transformerOut?: Action): this;
    /**
     * Add an alias for a given property.
     *
     * @param foreignPath The path of the incoming object.
     * @param localPath The path of the typed object's property.
     * @returns This builder.
     */
    alias(foreignPath: string, localPath: string): this;
    /**
     * Add a list item type to let the builder know how to construct
     * the elements of a list.
     *
     * @param localPath The path of the typed object's property.
     * @param builtObject The list element as a typed object.
     * @returns This builder.
     */
    listType(localPath: string, builtObject: ResultType): this;
    private getForeignObjectReference;
    fromJSON(json: any, strict?: boolean): ResultType;
    toJSON(source: ResultType): any;
}
//# sourceMappingURL=Builder.d.ts.map