"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var Describer_1 = __importDefault(require("./helpers/Describer"));
var BuildableResource_1 = __importDefault(require("./contracts/BuildableResource"));
var ResourceFactory_1 = __importDefault(require("./contracts/ResourceFactory"));
var JSONConvertible_1 = __importDefault(require("./contracts/JSONConvertible"));
var functions_1 = require("./helpers/functions");
var ObjectReference = /** @class */ (function () {
    function ObjectReference(path, value) {
        this.path = path;
        this.value = value;
    }
    return ObjectReference;
}());
/**
 * This is used to define how a class needs to be constructed from an object.
 */
var Builder = /** @class */ (function () {
    /**
     * Instantiates the given class thanks to a {@link ResourceFactory}.
     *
     * @param ctor The class that needs to be instantiated.
     */
    function Builder(ctor) {
        /**
         * List of parameters that need to be ignored.
         */
        this.ignores = new Set;
        /**
         * Collection of transformers. Each transformer is a callback function associated to a property,
         * this gets called just before the property is assigned to the {@link baseObject}
         */
        this.transformers = {};
        /**
         * Collection of aliases. Each alias represents an alternative name for the property
         * in the incoming object.
         */
        this.aliases = {};
        /**
         * Collection of list element constructors. This is used to define how to construct
         * the individual items of an incoming list.
         */
        this.listElementConstructors = {};
        this.baseObject = new ctor();
    }
    /**
     * Check if this builder is using a specific
     * class to build the object from.
     *
     * @param specifiedClass The class in question
     * @returns True if it's using it, false otherwise.
     */
    Builder.prototype.isUsingClass = function (specifiedClass) {
        return this.baseObject instanceof specifiedClass;
    };
    /**
     * Update base object of current builder and instantiates the given class thanks to a {@link ResourceFactory}.
     *
     * @param ctor The class that needs to be instantiated.
     */
    Builder.prototype.updateBaseObject = function (ctor) {
        this.baseObject = new ctor();
    };
    /**
     * Add an ignore directive for one or more paths.
     *
     * @param paths The object paths to ignore when instantiating.
     * @returns The builder.
     */
    Builder.prototype.ignore = function () {
        var _this = this;
        var paths = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            paths[_i] = arguments[_i];
        }
        paths.forEach(function (path) { return _this.ignores.add(path); });
        return this;
    };
    /**
     * Add a transform directive for a given path.
     *
     * @param localPath The property of the typed object that needs to be transformed.
     * @param transformer The {@link ValueTransformer} used.
     * @returns This builder.
     */
    Builder.prototype.transform = function (localPath, transformerIn, transformerOut) {
        var defaultTransformer = function (value) { return value; };
        this.transformers[localPath] = {
            in: transformerIn,
            out: transformerOut !== null && transformerOut !== void 0 ? transformerOut : defaultTransformer
        };
        return this;
    };
    /**
     * Add an alias for a given property.
     *
     * @param foreignPath The path of the incoming object.
     * @param localPath The path of the typed object's property.
     * @returns This builder.
     */
    Builder.prototype.alias = function (foreignPath, localPath) {
        this.aliases[localPath] = foreignPath;
        return this;
    };
    /**
     * Add a list item type to let the builder know how to construct
     * the elements of a list.
     *
     * @param localPath The path of the typed object's property.
     * @param builtObject The list element as a typed object.
     * @returns This builder.
     */
    Builder.prototype.listType = function (localPath, builtObject) {
        this.listElementConstructors[localPath] = builtObject;
        return this;
    };
    Builder.prototype.getForeignObjectReference = function (localPath, foreignObject) {
        var _a;
        if (foreignObject === void 0) { foreignObject = undefined; }
        var foreignPath = (_a = this.aliases[localPath]) !== null && _a !== void 0 ? _a : localPath;
        var foreignValue = (0, functions_1.dotAccess)(foreignPath, foreignObject);
        var result;
        if (foreignObject == undefined) {
            result = new ObjectReference(foreignPath, null);
        }
        else {
            result = new ObjectReference(foreignValue != null ? foreignPath : localPath, foreignValue);
        }
        result.value = (0, functions_1.dotAccess)(result.path, foreignObject);
        return result;
    };
    Builder.prototype.fromJSON = function (json, strict) {
        var _this = this;
        if (strict === void 0) { strict = false; }
        var target = this.baseObject;
        var params = Describer_1.default.getParameters(target);
        params.forEach(function (param) {
            if (_this.ignores.has(param) || !params.includes(param)) {
                return;
            }
            var foreignObject = _this.getForeignObjectReference(param, json);
            if (!json || !foreignObject.value) {
                if (strict) {
                    throw new Error("Invalid input object, missing parameter: " + param);
                }
                else
                    return;
            }
            if (target[param] instanceof BuildableResource_1.default) {
                target[param] = target[param].build.fromJSON(foreignObject.value, strict);
            }
            else if (Array.isArray(target[param])) {
                var list = foreignObject.value;
                target[param] = list.map(function (item, index) {
                    var listClassElement = _this.listElementConstructors[param];
                    if (listClassElement) {
                        var listElementClassBuilder = listClassElement.build;
                        return listElementClassBuilder.fromJSON(item, strict);
                    }
                    else if (!strict && foreignObject.value) {
                        return foreignObject.value[index];
                    }
                });
            }
            else {
                if (_this.transformers[param]) {
                    target[param] = _this.transformers[param].in(foreignObject.value);
                }
                else {
                    target[param] = foreignObject.value;
                }
            }
        });
        return (0, functions_1.deepCopy)(target);
    };
    Builder.prototype.toJSON = function (source) {
        var _this = this;
        var params = Describer_1.default.getParameters(source);
        var result = {};
        params.forEach(function (param) {
            var foreignObject = _this.getForeignObjectReference(param);
            if (_this.ignores.has(param)) {
                return;
            }
            if (source[param] instanceof BuildableResource_1.default) {
                result[foreignObject.path] = source[param].build.toJSON(source[param]);
            }
            else if (Array.isArray(source[param])) {
                var list = source[param];
                result[foreignObject.path] = list.map(function (item, index) {
                    var listClassElement = _this.listElementConstructors[param];
                    if (listClassElement) {
                        var listElementClassBuilder = listClassElement.build;
                        return listElementClassBuilder.toJSON(item);
                    }
                    else if (source.hasOwnProperty(param)) {
                        return _this.transformers[param] ? _this.transformers[param].out(result[foreignObject.path][index]) : result[foreignObject.path][index];
                    }
                });
            }
            else {
                result[foreignObject.path] = _this.transformers[param] ? _this.transformers[param].out(source[param]) : source[param];
            }
        });
        delete result["builder"]; // FIXME Magic number
        return result;
    };
    return Builder;
}());
exports.default = Builder;
//# sourceMappingURL=Builder.js.map